/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
// -------------------------------------------------------------------------------
// setupnvpair.cpp
//
// This file contains the NVPair class flattened into a set of C functions.  This
// is done to work around the problem of exporting C++ classes on NT which is a
// pain in the ass.  Trust me, I prefer C++ to C and only do this out of sheer
// necessity.
//
// All the methods of NVPair have been turned into C functions with the exception
// of the operator methods and the methods which use NSString as arguments.  
// Overall, this amounts to roughly four or five methods being removed which is 
// not a huge deal.
//
// I've pretty much done the same thing as a C++ compiler to get the NVPair object
// into C: I turned the object into an opaque type named NVPairHandle and transformed
// every method in NVPair into a function which takes NVPairHandle as the first
// argument.  All the functions do is cast NVPairHandle back into an NVPair object
// and invoke the method on that object which the function specifies.  Pretty simple.
//
// The names of the functions ironically match the NVPair method names with minor
// variations.  All the functions have the following name format:
//
//    setupNVPair[C++ Method Name]
//
// However, in certain cases this does not work, such as constructors, destructors
// and overloaded methods.  In the case of constructors, they are all here and
// begin with setupNVPairCreate followed by an extension to distinguish them.  The
// destructor is named setupNVPairDestroy.  The rest of the method names have been
// more or less kept with minor modification.  That is why the functions here are
// not documented.  See nsdefs.h for the NVPair class itself.
//
// You deal with an NVPairHandle in the same way as an NVPair object.  You must 
// first create it via one of the setupNVPairCreate methods.  After that you are
// free to use any of the setupNVPair functions.  When finished, you simply call
// setupNVPairDestroy to get rid of the NVPairHandle.  Keep in mind that the
// functions are lazy about making sure the given pointer is a legit NVPair object.  
// If you pass in a bad NVPairHandle don't expect anything good to happen.
//
// Author: Brian Hannan
//
// -------------------------------------------------------------------------------

#include "nsdefs.h"
#include "setupnvpair.h"


// A handy macro to cast the opaque void* NVPairHandle
// back into a NVPair object.
#define NVPairHandleToObject(x) ((NVPair*) x)


// -------------------------------------------------------------------------------
// setupNVPairCreateEmpty
// -------------------------------------------------------------------------------

NSAPI_EXTERN NVPairHandle 
setupNVPairCreateEmpty(void)
{
  return((NVPairHandle) (new NVPair()));
}


// -------------------------------------------------------------------------------
// setupNVPairCreateFromFile
// -------------------------------------------------------------------------------

NSAPI_EXTERN NVPairHandle
setupNVPairCreateFromFile(char *inFileName)
{
  return((NVPairHandle) (new NVPair(inFileName)));
}


// -------------------------------------------------------------------------------
// setupNVPairCreateFromNVPair
// -------------------------------------------------------------------------------

NSAPI_EXTERN NVPairHandle
setupNVPairCreateFromNVPair(void *inNVPair)
{
  return((NVPairHandle) (new NVPair(*NVPairHandleToObject(inNVPair))));
}


// -------------------------------------------------------------------------------
// setupNVPairDestroy
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairDestroy(NVPairHandle inNVPair)
{
  delete(NVPairHandleToObject(inNVPair));
}


// -------------------------------------------------------------------------------
// setupNVPairRead
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairRead(NVPairHandle inNVPair, char *inFileName)
{
  NVPairHandleToObject(inNVPair)->read(inFileName);
}


// -------------------------------------------------------------------------------
// setupNVPairIsEmpty
// -------------------------------------------------------------------------------

NSAPI_EXTERN Bool
setupNVPairIsEmpty(NVPairHandle inNVPair)
{
  return(NVPairHandleToObject(inNVPair)->isEmpty());
}


// -------------------------------------------------------------------------------
// setupNVPairClear
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairClear(NVPairHandle inNVPair)
{
  NVPairHandleToObject(inNVPair)->clear();
}


// -------------------------------------------------------------------------------
// setupNVPairGetID
// -------------------------------------------------------------------------------

NSAPI_EXTERN char*
setupNVPairGetID(NVPairHandle inNVPair)
{
  return((char*) (NVPairHandleToObject(inNVPair)->id()));
}


// -------------------------------------------------------------------------------
// setupNVPairSetID
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairSetID(NVPairHandle inNVPair, char *inID)
{
  NVPairHandleToObject(inNVPair)->id(inID);
}


// -------------------------------------------------------------------------------
// setupNVPairGetTimeStamp
// -------------------------------------------------------------------------------

NSAPI_EXTERN char*
setupNVPairGetTimeStamp(NVPairHandle inNVPair)
{
  return((char*) (NVPairHandleToObject(inNVPair)->timeStamp()));
}


// -------------------------------------------------------------------------------
// setupNVPairSetTimeStamp
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairSetTimeStamp(NVPairHandle inNVPair)
{
  NVPairHandleToObject(inNVPair)->stampTime();
}


// -------------------------------------------------------------------------------
// setupNVPairGet
// -------------------------------------------------------------------------------

NSAPI_EXTERN char*
setupNVPairGet(NVPairHandle inNVPair, char *inName)
{
  return((char*) (NVPairHandleToObject(inNVPair)->get(inName)));
}


// -------------------------------------------------------------------------------
// setupNVPairSetString
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairSetString(NVPairHandle inNVPair, char *inName, char *inValue)
{
  NVPairHandleToObject(inNVPair)->set(inName, inValue);
}


// -------------------------------------------------------------------------------
// setupNVPairSetInt
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairSetInt(NVPairHandle inNVPair, char *inName, int inValue)
{
  NVPairHandleToObject(inNVPair)->set(inName, inValue);
}


// -------------------------------------------------------------------------------
// setupNVPairSetUInt
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairSetUInt(NVPairHandle inNVPair, char *inName, unsigned int inValue)
{
  NVPairHandleToObject(inNVPair)->set(inName, inValue);
}


// -------------------------------------------------------------------------------
// setupNVPairSetLong
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairSetLong(NVPairHandle inNVPair, char *inName, long inValue)
{
  NVPairHandleToObject(inNVPair)->set(inName, inValue);
}


// -------------------------------------------------------------------------------
// setupNVPairSetULong
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairSetULong(NVPairHandle  inNVPair,  char *inName, unsigned long inValue)
{
  NVPairHandleToObject(inNVPair)->set(inName, inValue);
}


// -------------------------------------------------------------------------------
// setupNVPairSetSringBySection
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairSetStringBySection(NVPairHandle inNVPair, char *inSection, char *inName, char *inValue)
{
  NVPairHandleToObject(inNVPair)->set(inSection, inName, inValue);
}


// -------------------------------------------------------------------------------
// setupNVPairSetLongBySection
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairSetLongBySection(NVPairHandle inNVPair, char *inSection,  char *inName, long inValue)
{
  NVPairHandleToObject(inNVPair)->set(inSection, inName, inValue);
}


// -------------------------------------------------------------------------------
// setupNVPairGetListItem
// -------------------------------------------------------------------------------

NSAPI_EXTERN char*
setupNVPairGetListItem(NVPairHandle inNVPair, char *inName, int inIndex)
{
  return((char*) (NVPairHandleToObject(inNVPair)->getListItem(inName, inIndex)));
}


// -------------------------------------------------------------------------------
// setupNVPairGetListItems
// -------------------------------------------------------------------------------

NSAPI_EXTERN char**
setupNVPairGetListItems(NVPairHandle inNVPair, char *inName)
{
  return(NVPairHandleToObject(inNVPair)->getListItems(inName));
}


// -------------------------------------------------------------------------------
// setupNVPairFreeListItems
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairFreeListItems(NVPairHandle inNVPair, char **inItems)
{
  NVPairHandleToObject(inNVPair)->freeListItems(inItems);
}


// -------------------------------------------------------------------------------
// setupNVPairInList
// -------------------------------------------------------------------------------

NSAPI_EXTERN Bool
setupNVPairInList(NVPairHandle inNVPair, char *inName, char *inValue)
{
  return(NVPairHandleToObject(inNVPair)->inList(inName, inValue));
}


// -------------------------------------------------------------------------------
// setupNVPairListExists
// -------------------------------------------------------------------------------

NSAPI_EXTERN Bool
setupNVPairListExists(NVPairHandle inNVPair, char *inName)
{
  return(NVPairHandleToObject(inNVPair)->listExists(inName));
}


// -------------------------------------------------------------------------------
// setupNVPairAddString
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairAddString(NVPairHandle inNVPair, char *inList, char *inValue)
{
  NVPairHandleToObject(inNVPair)->add(inList, inValue);
}


// -------------------------------------------------------------------------------
// setupNVPairAddStrings
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairAddStrings(NVPairHandle inNVPair, char *inList, char **inValues)
{
  NVPairHandleToObject(inNVPair)->add((const char*) inList, (const char**) inValues);
}


// -------------------------------------------------------------------------------
// setupNVPairAddStringBySection
// -------------------------------------------------------------------------------

NSAPI_EXTERN void 
setupNVPairAddStringBySection(NVPairHandle inNVPair, char *inSection, char *inList, char *inValue)
{
  NVPairHandleToObject(inNVPair)->add(inSection, inList, inValue);
}


// -------------------------------------------------------------------------------
// setupNVPairAddStringsBySection
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairAddStringsBySection(NVPairHandle inNVPair, char *inSection, char *inList, char **inValues)
{
  NVPairHandleToObject(inNVPair)->add((const char*) inSection, (const char*) inList, (const char**) inValues);
}


// -------------------------------------------------------------------------------
// setupNVPairDelList
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairDelList(NVPairHandle  inNVPair, char *inName)
{
  NVPairHandleToObject(inNVPair)->delList(inName);
}


// -------------------------------------------------------------------------------
// setupNVPairDelListItem
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairDelListItem(NVPairHandle inNVPair, char *inName, char *inItem)
{
  NVPairHandleToObject(inNVPair)->delListItem(inName, inItem);
}


// -------------------------------------------------------------------------------
// setupNVPairNumName
// -------------------------------------------------------------------------------

NSAPI_EXTERN int
setupNVPairNumName(NVPairHandle inNVPair)
{
  return(NVPairHandleToObject(inNVPair)->numName());
}


// -------------------------------------------------------------------------------
// setupNVPairNumList
// -------------------------------------------------------------------------------

NSAPI_EXTERN int
setupNVPairNumList(NVPairHandle inNVPair)
{
  return(NVPairHandleToObject(inNVPair)->numList());
}


// -------------------------------------------------------------------------------
// setupNVPairNumListItems
// -------------------------------------------------------------------------------

NSAPI_EXTERN int
setupNVPairNumListItems(NVPairHandle inNVPair, char *inList)
{
  return(NVPairHandleToObject(inNVPair)->numListItems(inList));
}


// -------------------------------------------------------------------------------
// setupNVPairName
// -------------------------------------------------------------------------------

NSAPI_EXTERN char*
setupNVPairName(NVPairHandle inNVPair, int inIndex)
{
  return((char*) (NVPairHandleToObject(inNVPair)->name(inIndex)));
}


// -------------------------------------------------------------------------------
// setupNVPairList
// -------------------------------------------------------------------------------

NSAPI_EXTERN char*
setupNVPairList(NVPairHandle inNVPair, int inIndex)
{
  return((char*) (NVPairHandleToObject(inNVPair)->list(inIndex)));
}


// -------------------------------------------------------------------------------
// setupNVPairNumSections
// -------------------------------------------------------------------------------

NSAPI_EXTERN int
setupNVPairNumSections(NVPairHandle inNVPair)
{
  return(NVPairHandleToObject(inNVPair)->numSections());
}


// -------------------------------------------------------------------------------
// setupNVPairGetSectionByIndex
// -------------------------------------------------------------------------------

NSAPI_EXTERN NVPairHandle
setupNVPairGetSectionByIndex(NVPairHandle inNVPair, int inIndex)
{
  return((NVPairHandle) (NVPairHandleToObject(inNVPair)->getSection(inIndex)));
}


// -------------------------------------------------------------------------------
// setupNVPairGetSectionByName
// -------------------------------------------------------------------------------

NSAPI_EXTERN NVPairHandle
setupNVPairGetSectionByName(NVPairHandle inNVPair, char *inName)
{
  return((NVPairHandle) (NVPairHandleToObject(inNVPair)->getSection(inName)));
}


// -------------------------------------------------------------------------------
// setupNVPairAddSection
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairAddSection(NVPairHandle inNVPair, char *inName,  NVPairHandle inSubBlock)
{
  NVPairHandleToObject(inNVPair)->addSection(inName, NVPairHandleToObject(inSubBlock));
}


// -------------------------------------------------------------------------------
// setupNVPairCreateSection
// -------------------------------------------------------------------------------

NSAPI_EXTERN NVPairHandle
setupNVPairCreateSection(NVPairHandle inNVPair, char *inName)
{
  return((NVPairHandle) (NVPairHandleToObject(inNVPair)->createSection(inName)));
}


// -------------------------------------------------------------------------------
// setupNVPairRename
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairRename(NVPairHandle inNVPair, char *inOldLabel, char *inNewLabel)
{
  NVPairHandleToObject(inNVPair)->rename(inOldLabel, inNewLabel);
}


// -------------------------------------------------------------------------------
// setupNVPairRemove
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairRemove(NVPairHandle inNVPair, char *inLabel)
{
  NVPairHandleToObject(inNVPair)->remove(inLabel);
}


// -------------------------------------------------------------------------------
// setupNVPairToUTF8
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairToUTF8(NVPairHandle inNVPair, char *inLabel)
{
  NVPairHandleToObject(inNVPair)->toUTF8(inLabel);
}


// -------------------------------------------------------------------------------
// setupNVPairToLocal
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairToLocal(NVPairHandle inNVPair, char *inLabel)
{
  NVPairHandleToObject(inNVPair)->toLocal(inLabel);
}


// -------------------------------------------------------------------------------
// setupNVPairWriteToFile
// -------------------------------------------------------------------------------

NSAPI_EXTERN int
setupNVPairWriteToFile(NVPairHandle inNVPair, char *inFileName)
{
  return(NVPairHandleToObject(inNVPair)->write(inFileName));
}


// -------------------------------------------------------------------------------
// setupNVPairWrite
// -------------------------------------------------------------------------------

NSAPI_EXTERN int
setupNVPairWrite(NVPairHandle inNVPair)
{
  return(NVPairHandleToObject(inNVPair)->write());
}


// -------------------------------------------------------------------------------
// setupNVPairSetFormat
// -------------------------------------------------------------------------------

NSAPI_EXTERN void
setupNVPairSetFormat(NVPairHandle inNVPair, int inWhich)
{
  NVPairHandleToObject(inNVPair)->setFormat(inWhich);
}

